//@flow

import {whitespace} from './fixtures'

export /*::opaque*/
type FinitePositiveInt/*: number*/ = number

export class Ident {
  ident: FinitePositiveInt
  inspect() {
    return `Ident ${this.ident}`
  }
  // equals(b: Ident) {
  //   return equals(this, b)
  // }
  // lt(b: Ident) {
  //   return lt(this, b)
  // }
  // plus(b: Ident) {
  //   return plus(this, b)
  // }
  // toString() {
  //   return toString(this)
  // }
}

const cache: Map<FinitePositiveInt, Ident> = new Map

function clamp(ident: number): FinitePositiveInt {
  if (isNaN(ident)) return 0
  if (!isFinite(ident)) return 0
  if (ident < 0) return 0
  return ident
}

export function create(identSize: number): Ident {
  const ident = clamp(identSize)
  const cached = cache.get(ident)
  if (cached) return cached
  const result = new Ident
  result.ident = ident
  cache.set(ident, result)
  return result
}

export function equals(a: Ident, b: Ident) {
  if (a === b) return true
  if (a.ident === b.ident) return true
  return false
}

export function lt(a: Ident, b: Ident): 0 | 1 | -1 {
  if (equals(a, b)) return 0
  if (a.ident > b.ident) return 1
  return -1
}

export function zero() {
  return create(0)
}

export function isZero(ident: Ident) {
  return ident.ident <= 0
}

export function plus(a: Ident, b: Ident) {
  return create(a.ident + b.ident)
}

export function fromString(str: string) {
  const ident = str.length - str.trimLeft().length
  return create(ident)
}

export function toString(ident: Ident) {
  return whitespace(ident.ident)
}
