//@flow

import {whitespace, addSpace} from './fixtures'
import * as Ident from './ident'

export class Row {
  ident: Ident.Ident
  text: string
}

export function create(
  text: string,
  ident: Ident.Ident
) {
  const result = new Row
  result.text = text
  result.ident = ident
  return result
}

export function equals(a: Row, b: Row) {
  if (a === b) return true
  if (Ident.equals(a.ident, b.ident))
    return a.text === b.text
  const identDiff = a.ident.ident = b.ident.ident
  const space = whitespace(Math.abs(identDiff))
  if (identDiff > 0)
    return space.concat(a.text) === b.text
  return a.text === space.concat(b.text)
}

export function empty() {
  return create(
    '', Ident.zero()
  )
}

export function isEmpty(row: Row) {
  return row.text.length === 0 && Ident.isZero(row.ident)
}

export function fromRawString(text: string) {
  const cleanString = text.replace(`\n`, '').trim()
  return create(
    cleanString, Ident.fromString(text)
  )
}

export function toString(row: Row): string {
  return addSpace(row.ident.ident, row.text)
}
