//@flow

import {addSpace} from './fixtures'

import * as Row from './row'
import * as Ident from './ident'

export class SemiRow {
  ident: Ident.Ident
  text: string[]
  inspect() {
    return `SemiRow ${this.ident.ident} {${this.text.join('|')}}`
  }
}

export function create(
  ident: Ident.Ident,
  text: string[]
): SemiRow {
  const result = new SemiRow
  result.ident = ident
  result.text = text
  return result
}

export function appendStrings(
  str: string[],
  row: SemiRow
): SemiRow {
  return create(
    row.ident,
    row.text.concat(str)
  )
}

export type PlainRow = Row.Row

export function toPlainRows(row: SemiRow): Array<Row.Row> {
  return row.text.map(
    str => Row.create(str, row.ident)
  )
}

export function fromRow(row: Row.Row): SemiRow {
  return create(row.ident, [row.text])
}

export function empty(): SemiRow {
  return create(Ident.zero(), [])
}

export function fromRawString(text: string): SemiRow {
  return fromRow(Row.fromRawString(text))
}

export function toString(row: SemiRow) {
  const {ident, text} = row
  return text
    .map(str => addSpace(ident.ident, str))
    .join(`\n`)
}

