//@flow

import * as Row from './semi-row'
// import * as PlainRow from './row'
import * as Ident from './ident'

export class Text {
  rows: Array<Row.SemiRow>
}

export function create(rows: Array<Row.SemiRow>): Text {
  const result = new Text
  result.rows = rows
  return result
}

export function empty(): Text {
  return create([])
}

export function concat(a: Text, b: Text) {
  return create(a.rows.concat(b.rows))
}

export function appendRow(text: Text, row: Row.SemiRow): Text {
  if (text.rows.length === 0)
    return create([row])
  const lastRow = text.rows[text.rows.length - 1]
  if (Ident.equals(row.ident, lastRow.ident)) {
    // let newLastRow
    // if (Row)
    const newLastRow = Row.appendStrings(row.text, lastRow)
    // console.log(text.rows)
    // console.log(text.rows
    //   .slice(0, -1))
    const rows = text.rows
      .slice(0, -1)
      .concat([newLastRow])
    return create(rows)
  }
  return create(text.rows.concat([row]))
}

export function compact(text: Text): Text {
  const flatRows = toPlainRows(text)
    .reduce((acc: Array<Row.PlainRow>, row): Array<Row.PlainRow> => {
      const str = row.text
      if (str.length > 0) return [...acc, row]
      if (acc.length === 0) return [row]
      const last = acc[acc.length - 1]
      if (last.text.length > 0) return [...acc, row]

      return acc
        .slice(0, -1)
        .concat([row])
    }, [])

  const cleanRows = flatRows.map(row => Row.create(
    row.ident,
    [row.text]
  ))
  return cleanRows.reduce((acc, val) => appendRow(acc, val), empty())
}


// function joinSpaces(text: string[]): string[] {
//   return text.reduce(
//     (acc: string[], str): string[] => {
//       if (str.length > 0) return [...acc, str]
//       if (acc.length === 0) return [str]
//       const last = acc[acc.length - 1]
//       if (last.length > 0) return [...acc, str]

//       return acc
//         .slice(0, -1)
//         .concat([str])
//     }, ([]: string[])
//   )
// }

export function toPlainRows(text: Text) {
  return text.rows
    .reduce((acc: Array<Row.PlainRow>, row) => acc.concat(
      Row.toPlainRows(row)
    ), [])
}

export function fromString(rawString: string): Text {
  const strings = rawString.split(`\n`)
  const rows = strings.map(str => Row.fromRawString(str))
  const rawResult = rows.reduce((acc, val) => appendRow(acc, val), empty())
  return compact(rawResult)
}

export function toString(text: Text) {
  return text.rows
    .map(Row.toString)
    .join(`\n`)
}
